﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using AIDemos;
using Microsoft.Xna.Framework.Content;
using System.Threading;
using Microsoft.Xna.Framework.Input;
using Brains.Framework;
using AIRendering;
using AIDemos.GameClasses;
using Brains;
using Brains.Framework.Behaviors;
using AIDemos.GameClasses.Behaviors;
using System.Collections;
using Brains.Framework.Locomotion;

namespace AIDemos.Screens.Demos
{
    public class Demo4 : DemoGameScreen
    {
        
        public Demo4():base("Demo 4 - Steering Motion Controller","")
        {
            TransitionOnTime = TimeSpan.FromSeconds(1.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);
            
            
        }

        public override void LoadContent()
        {
            base.LoadContent();
            
            LoadAI();

            CenterCamOnMap();
            ScreenManager.Game.ResetElapsedTime();
        }
        private void LoadAI()
        {
            
            _engine.CreateWorld(new DrawableWorld());
            _engine.World.LoadMapDataFromTexture(
                content.Load<Texture2D>("DemoTextures/MAPDemoBlank8"),
                64,
                typeof(DrawableGrid));



            _actor = new GameActor(new Vector2(32, 32), 5);
            _actor.Locomotion = new LocomtionSteering();
            _actor.Locomotion.MaxSpeed = 50;
            _actor.Locomotion.MaxRotation = MathHelper.TwoPi;
            _engine.World.AddActor(_actor);

            SwitchMode(DemoMode.Seek);
        }

        
        GameActor _actor;
       
        public override void UnloadContent()
        {
            content.Unload();
        }

        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                     bool coveredByOtherScreen)
        {
            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

            if (IsActive)
            {
                _engine.Update(gameTime);
            }
        }
        MouseState _currentMouse;
        MouseState _lastMouse;
        public override void HandleInput(InputState input)
        {
            if (input == null)
                throw new ArgumentNullException("input");

            _lastMouse = _currentMouse;
            _currentMouse = Mouse.GetState();


            int playerIndex = (int)ControllingPlayer.Value;
            
            KeyboardState keyboardState = input.CurrentKeyboardStates[playerIndex];
            GamePadState gamePadState = input.CurrentGamePadStates[playerIndex];

            bool gamePadDisconnected = !gamePadState.IsConnected &&
                                       input.GamePadWasConnected[playerIndex];

            if (!gamePadDisconnected)
            {
                Vector2 mousPos = new Vector2(_currentMouse.X, _currentMouse.Y);
                Vector2 mapMousePos = mousPos + primitiveBatch.CameraPosition;
                   
                if (IsNewLeftMouseClick())
                {
                    SetSeekPosition(mapMousePos);
                }

                if (IsNewKeyPress(input, Keys.F1))
                    SwitchMode(DemoMode.Seek);
                
                if (IsNewKeyPress(input,Keys.F2))
                    SwitchMode(DemoMode.Arrive);
                if (IsNewKeyPress(input, Keys.F3))
                    SwitchMode(DemoMode.Wander);  
                
            }

            base.HandleInput(input);
        }

        private void SetSeekPosition(Vector2 mapMousePos)
        {
            ((LocomtionSteering)_actor.Locomotion).SeekPos = mapMousePos;
        }

        DemoMode _currentMode;
        private void SwitchMode(DemoMode demoMode)
        {
            _currentMode = demoMode;
            ((LocomtionSteering)_actor.Locomotion).TurnAllOff();
            switch (demoMode)
            {
                case DemoMode.Seek:
                    ((LocomtionSteering)_actor.Locomotion).TurnOnSeek(new Vector2(400));
                    break;
                case DemoMode.Arrive:
                    ((LocomtionSteering)_actor.Locomotion).TurnOnArrive(new Vector2(400));
                    break;
                case DemoMode.Wander:
                    ((LocomtionSteering)_actor.Locomotion).TurnOnWander();
                    break;
            }
        }
        
        enum DemoMode
        {
            Seek,
            Arrive,
            Wander
        }
        private bool IsNewKeyPress(InputState input, Keys keys)
        {
            return input.CurrentKeyboardStates[0].IsKeyUp(keys) &&
                input.LastKeyboardStates[0].IsKeyDown(keys);
        }

        private bool IsNewLeftMouseClick()
        {
            return _currentMouse.LeftButton == ButtonState.Released &&
                _lastMouse.LeftButton == ButtonState.Pressed;
        }

        public override void Draw(GameTime gameTime)
        {
            ScreenManager.GraphicsDevice.Clear(ClearOptions.Target,
                                               Color.Black, 0, 0);

            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;

            spriteBatch.Begin();
            spriteBatch.DrawString(
                gameFont,
                "Mode: " + _currentMode.ToString(),
                new Vector2(ScreenManager.GraphicsDevice.Viewport.Width - 200, 10),
                Color.White,
                0,
                new Vector2(0),
                0.5f,SpriteEffects.None,
                0);

            base.Draw(gameTime);
            ((IRender)_engine.World).Render(primitiveBatch);
            spriteBatch.End();

            primitiveBatch.DrawCircle(((LocomtionSteering)_actor.Locomotion).SeekPos, 5, Color.Red);
            
            if (TransitionPosition > 0)
                ScreenManager.FadeBackBufferToBlack(255 - TransitionAlpha);
        }
    }
}
