#region File Description
//-----------------------------------------------------------------------------
// MainMenuScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using Microsoft.Xna.Framework;
using AIDemos.Screens.Demos;
#endregion

namespace AIDemos
{
    /// <summary>
    /// The main menu screen is the first thing displayed when the game starts up.
    /// </summary>
    class MainMenuScreen : MenuScreen
    {
        


        /// <summary>
        /// Constructor fills in the menu contents.
        /// </summary>
        public MainMenuScreen()
            : base("Main Menu")
        {
            // Create our menu entries.
            MenuEntry playGameMenuEntry = new MenuEntry("Demo1 - Basic Pathfinding");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo2 - Patrol");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo3 - Custom Behaviors");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo4 - Steering Motion Controller");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo5 - A Bigger World");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo6 - Combining Behaviors");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);

            playGameMenuEntry = new MenuEntry("Demo7 - Group Movement");
            playGameMenuEntry.Selected += PlayGameMenuEntrySelected;
            MenuEntries.Add(playGameMenuEntry);
            
            MenuEntry optionsMenuEntry = new MenuEntry("Options");
            MenuEntry exitMenuEntry = new MenuEntry("Exit");

            // Hook up menu event handlers.

            optionsMenuEntry.Selected += OptionsMenuEntrySelected;
            exitMenuEntry.Selected += OnCancel;

            // Add entries to the menu.

            MenuEntries.Add(optionsMenuEntry);
            MenuEntries.Add(exitMenuEntry);
        }



     

        /// <summary>
        /// Event handler for when the Play Game menu entry is selected.
        /// </summary>
        void PlayGameMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            switch (((MenuEntry)sender).Text)
            {
                case "Demo1 - Basic Pathfinding":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo1());
                    break;
                case "Demo2 - Patrol":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo2());
                    break;
                case "Demo3 - Custom Behaviors":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo3());
                    break;
                case "Demo4 - Steering Motion Controller":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo4());
                    break;
                case "Demo5 - A Bigger World":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo5());
                    break;
                case "Demo6 - Combining Behaviors":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo6());
                    break;
                case "Demo7 - Group Movement":
                    LoadingScreen.Load(ScreenManager, true, e.PlayerIndex,
                               new Demo7());
                    break;
            }
        }


        /// <summary>
        /// Event handler for when the Options menu entry is selected.
        /// </summary>
        void OptionsMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            ScreenManager.AddScreen(new OptionsMenuScreen(), e.PlayerIndex);
        }


        /// <summary>
        /// When the user cancels the main menu, ask if they want to exit the sample.
        /// </summary>
        protected override void OnCancel(PlayerIndex playerIndex)
        {
            const string message = "Are you sure you want to exit this sample?";

            MessageBoxScreen confirmExitMessageBox = new MessageBoxScreen(message);

            confirmExitMessageBox.Accepted += ConfirmExitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmExitMessageBox, playerIndex);
        }


        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure
        /// you want to exit" message box.
        /// </summary>
        void ConfirmExitMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {
            ScreenManager.Game.Exit();
        }


    }
}
