﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Brains.Framework.Designer;
using Brains.Framework.Behaviors;

namespace Designer.Classes
{
    public partial class BehaviorTreeViewer : UserControl
    {
        public BehaviorNode rootBehavior;
        public BehaviorTreeViewer()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            e.Graphics.Clear(Color.White);
            if(rootBehavior!=null)
                DrawRootBehavior(e,rootBehavior);
            
            base.OnPaint(e);
        }

        private void DrawRootBehavior(PaintEventArgs e, BehaviorNode rootBehavior)
        {
            Point p = new Point(Width / 2, 50);
            
            DrawBehavior(e, rootBehavior, p);
        }

        private void DrawBehavior(PaintEventArgs e, BehaviorNode rootBehavior, Point p)
        {
            int verticalgap=50;
            int horizgap=50;
            
            
            if (rootBehavior.Type == null)
            {
                Rectangle _r = new Rectangle(p.X - 16, p.Y - 16, 32, 32);
                e.Graphics.FillRectangle(Brushes.LightGray, _r);
                e.Graphics.DrawRectangle(Pens.Black, _r);
            }
            if (rootBehavior.Type == typeof(SequenceBehavior))
            {
                Rectangle _r = new Rectangle(p.X - 16, p.Y - 16, 32, 32);
                e.Graphics.FillRectangle(Brushes.LightGray, _r);
                e.Graphics.DrawRectangle(Pens.Black, _r);
            }
            if (rootBehavior.Type == typeof(SelectorBehavior))
            {
                Rectangle _r = new Rectangle(p.X - 16, p.Y - 16, 32, 32);
                e.Graphics.FillEllipse(Brushes.LightGray, _r);
                e.Graphics.DrawEllipse(Pens.Black, _r);
                e.Graphics.DrawString("?", SystemFonts.CaptionFont, Brushes.Black, p);
            }
            if (rootBehavior.Type == typeof(RandomBehavior))
            {
                Rectangle _r = new Rectangle(p.X - 16, p.Y - 16, 32, 32);
                e.Graphics.FillEllipse(Brushes.LightGray, _r);
                e.Graphics.DrawEllipse(Pens.Black, _r);
                e.Graphics.DrawString("R", SystemFonts.CaptionFont, Brushes.Black, p);
            }
            

            if (rootBehavior.Type!=null  && rootBehavior.Type.BaseType == typeof(BehaviorTask))
            {
                Rectangle _r = new Rectangle(p.X - 16, p.Y - 16, 32, 32);
                e.Graphics.FillEllipse(Brushes.Orange, _r);
                e.Graphics.DrawEllipse(Pens.Black, _r);
                
            }
            Point subPosition=p;
            subPosition.Y+=verticalgap;
            
            if(rootBehavior.SubBehaviors.Count>1)
                subPosition.X-=horizgap*(rootBehavior.SubBehaviors.Count/2);
    
            foreach (var item in rootBehavior.SubBehaviors)
            {
                DrawBehavior(e, item, subPosition);
                //Draw Connector
                e.Graphics.DrawLine(Pens.Black,
                    p.X, p.Y + 16,
                    subPosition.X, subPosition.Y - 16);
                subPosition.X += horizgap;
            }

            if (rootBehavior.BehaviorName != null)
            {
                Point namePos = p;
                namePos.Y -= 14;
                namePos.X -= (int)(
                    e.Graphics.MeasureString(rootBehavior.BehaviorName,
                    SystemFonts.CaptionFont,
                    500).Width / 2);
                e.Graphics.DrawString(rootBehavior.BehaviorName,
                    SystemFonts.CaptionFont,
                    Brushes.Black,
                    namePos);
            }
        }

        private void DrawSequenceNode(PaintEventArgs e,Rectangle rect)
        {
            e.Graphics.FillRectangle(Brushes.LightGray, rect);
            e.Graphics.DrawRectangle(Pens.Black, rect);
        }
    }
}
